#include "CliTest.h"

#include <iostream>

#include <lexer/Lexer.h>
#include <parser/Parser.h>

#include <primitive/Integer.h>

#include <sys/stat.h>
#include <abstraction/AlgorithmRegistry.hpp>
#include <abstraction/XmlFileWriterRegistry.hpp>
#include <abstraction/XmlParserRegistry.hpp>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( CliTest, "common" );
CPPUNIT_TEST_SUITE_REGISTRATION ( CliTest );

void CliTest::setUp ( ) {
}

void CliTest::tearDown ( ) {
}

class One {
public:
	static int one ( ) {
		return 1;
	}
};

class Two {
public:
	static int two ( ) {
		return 2;
	}
};

class Add {
public:
	static int add ( int a, const int & b ) {
		return a + b;
	}

	static int add2 ( const primitive::Integer & a, int b ) {
		return a.getData ( ) + b;
	}
};

class Neg {
public:
	static int neg ( int a ) {
		return - a;
	}
};

class IntToDouble {
public:
	static double cast ( int a ) {
		return a;
	}
};

class Divide {
public:
	static double divide ( double a, double b ) {
		return a / b;
	}

	static int divide2 ( int a, int b ) {
		return a / b;
	}
};

void CliTest::testCreateUnique ( ) {
	abstraction::AlgorithmRegistry::registerAlgorithm < One > ( One::one, false, false );
	abstraction::AlgorithmRegistry::registerAlgorithm < Two > ( Two::two, false, false );
	abstraction::AlgorithmRegistry::registerAlgorithm < Add > ( Add::add, false, false );
	abstraction::AlgorithmRegistry::registerAlgorithm < Add > ( Add::add2, false, false );
	abstraction::AlgorithmRegistry::registerAlgorithm < Neg > ( Neg::neg, false, false );
	abstraction::AlgorithmRegistry::registerAlgorithm < Divide > ( Divide::divide, false, false );
	abstraction::AlgorithmRegistry::registerAlgorithm < Divide > ( Divide::divide2, false, false );

	mkdir ( "local", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH );

	// execute automaton::Determinize < aaa.xml <( parseString aaa.text ) <(decodeTree aaa.raw) > res.xml // determinizuj, parametry podle ocekavaneho typu z xml, string z textove reprezentace, strom z raw reprezentace; zapis jako xml do souboru
	// execute automaton::RandomAutomaton DFA 2 2 2 // nahodny automat DFA s danymi vlastnostmi, vypis na konzoli
	// execute string::Normalize < str.xml | string::PatternMatch - < pattern.xml > match_result.xml // - je predchozi vysledek

	try {
		cli::Environment environment;
		environment.setBinding ( "1", "1" );
		cli::Parser parser ( cli::Lexer ( "execute One | Add <( Add (int) #1 <(One) ) - | Neg - > local/xxx.xml" ) );
		parser.parse ( )->run ( environment );

		environment.setBinding ( "2", "local/xxx.xml" );
		parser = cli::Parser ( cli::Lexer ( "execute One | Add <( Add (int) <#2 <(One) ) - | Neg (double) - | Divide (double) - <(One | (double) Add <(One) - )" ) );
		parser.parse ( )->run ( environment );

		parser = cli::Parser ( cli::Lexer ( "execute One > $res" ) );
		parser.parse ( )->run ( environment );

		parser = cli::Parser ( cli::Lexer ( "execute $res" ) );
		parser.parse ( )->run ( environment );
	} catch ( const char * ex ) {
		std::cout << ex << std::endl;
	}

}

class Source {
public:
	static std::unique_ptr < int > source ( ) {
		return std::make_unique < int > ( 1 );
	}
};

class Sink {
public:
	static void sink ( std::unique_ptr < int > val ) {
		std::cout << * val << std::endl;
	}
};

void CliTest::testMove ( ) {
	abstraction::AlgorithmRegistry::registerAlgorithm < Source > ( Source::source, false, false );
	abstraction::AlgorithmRegistry::registerAlgorithm < Sink > ( Sink::sink, false, false );

	try {
		cli::Environment environment;
		cli::Parser parser ( cli::Lexer ( "execute Source | Sink ^ - >" ) );
		parser.parse ( )->run ( environment );
	} catch ( const char * ex ) {
		std::cout << ex << std::endl;
	}

}

